/*
 * SmartcardioNFC - a library providing a smartcardio implementation for NFC on Android
 *
 * Copyright (C) 2017 gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH <info@gematik.de> +49 30 40041-0
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.gematik.security.test;

import android.content.Context;
import android.nfc.NfcAdapter;
import android.util.Log;

import java.lang.reflect.Method;

public class NfcControlThread implements Runnable {
    private static final String TAG = NfcControlThread.class.getName();

    private NfcHealthcardTestActivity activity = null;

    public NfcControlThread(NfcHealthcardTestActivity activity) {
        this.activity = activity;
    }

    public void run() throws RuntimeException {
        try {
            Log.i(TAG, "Waiting 3 sec before disabling/enabling NFC...");
            Thread.sleep(3000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        try {
            if (activity.hasDiscoveredCard()) {
                Log.i(TAG, "Do not disable/enable NFC because card has already been discovered");
            } else {
                Log.i(TAG, "Now disable/enable NFC...");
                if (!powerNfc(false, activity)) {
                    throw new RuntimeException("Disabling NFC failed!");
                }
                Log.i(TAG, "NFC successfully disabled.");
                if (!powerNfc(true, activity)) {
                    throw new RuntimeException("Enabling NFC failed!");
                }
                Log.i(TAG, "NFC successfully enabled.");
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public boolean powerNfc(boolean isOn, Context context) throws Exception {
        boolean success = false;
        NfcAdapter nfcAdapter = NfcAdapter.getDefaultAdapter(context);

        if (nfcAdapter != null) {
            Class<?> NfcManagerClass;
            Method setNfcEnabled;

            NfcManagerClass = Class.forName(nfcAdapter.getClass().getName());
            setNfcEnabled = NfcManagerClass.getDeclaredMethod(isOn
                    ? "enable" : "disable");
            setNfcEnabled.setAccessible(true);
            success = (Boolean) setNfcEnabled.invoke(nfcAdapter);
        }

        return success;
    }

}
